******* Do file accompanying presentation by Sophia Rabe-Hesketh *******
******* 8th German Stata Users Group Meeting, Berlin, June, 2010 *******

******* Relevant paper:
/*
Rabe-Hesketh, S. Skrondal, A. and Gjessing, H. K. (2008). 
Biometrical modeling of twin and family data using standard 
mixed model software.  Biometrics 64 (1), 280-288. 
*/

******* Datasets (see slides for references):
/*
twin_bmi.dta
twin_neur.dta
twin_hay.dta
fam_birthwt.dta
*/

version 11.1


******* Continuous heights, P1 ACE, AE *************************************
** (P1 requires equality constraints between variances at 
**      different levels, so xtmixed cannot be used)

use twin_bmi, clear

list pair M member male height if s==1, sepby(pair) noobs

generate var3 = M + sqrt(1/2)*(1-M)
generate var2 = sqrt(1/2)*(1-M)

eq var3: var3
eq var2: var2 
generate one = 1
eq cons: one

cons def 1 [mem1_1]var2 = [pai2_1]var3

* takes some time (slowest model in do file)
gllamm height male, i(member pair) nrf(1 2) eqs(var2 var3 cons) nocor ///
    constr(1) adapt
	
* heritability:

display [mem1_1]var2^2/( [mem1_1]var2^2+[pai2_2]one^2+exp(2*[lns1]_cons))

* (cannot use _diparm for confidence interval because parameters
*   are not [eqname]_cons)	
	
	
******* P1 AE 

matrix a=e(b)
gllamm height male, i(member pair) nrf(1 1) eqs(var2 var3) nocor ///
    constr(1) adapt from(a) long skip

* heritability:

display [mem1_1]var2^2/( [mem1_1]var2^2+exp(2*[lns1]_cons))

	

******** Continuous heights, P2 ACE ****************************************

use twin_bmi, clear

generate k = pair if M==1
replace k = member if M==0

xtmixed height male || pair: || k:, mle variance

nlcom (var_A: 2*exp(2*[lns2_1_1]_cons)) ///
      (var_C: exp(2*[lns1_1_1]_cons)-exp(2*[lns2_1_1]_cons))
	 
* heritability
display 2*exp(2*[lns2_1_1]_cons)/(exp(2*[lns1_1_1]_cons)+exp(2*[lns2_1_1]_cons)+exp(2*[lnsig_e]_cons))

* approximate confidence intervals for variance components and heritability
* please check derivatives!
capture program drop _all
program define ciherit
  *ml display, first plus
  _coef_table, first plus
 noi _diparm lns2_1_1, label(var_A)                          ///
  func(2*exp(2*@)) deriv(4*exp(2*@))                         ///
  level(95) ci(log) 
 noi _diparm lns1_1_1 lns2_1_1, label(var_C)                 ///
  func(exp(2*@1)-exp(2*@2)) deriv(2*exp(2*@1)  -2*exp(2*@2)) ///
  level(95) ci(log) 
 noi _diparm lnsig_e, label(var_E)                           ///
  func(exp(2*@)) deriv(2*exp(2*@))                           ///
  level(95) ci(log) 
 noi _diparm lns1_1_1 lns2_1_1 lnsig_e, label(heritability)      ///
  func(2*exp(2*@2)/(exp(2*@1)+exp(2*@2)+exp(2*@3)))              ///
  deriv(-4*exp(2*@1+2*@2)/(exp(2*@1)+exp(2*@2)+exp(2*@3))^2 ///
	  -4*exp(4*@2)/(exp(2*@1)+exp(2*@2)+exp(2*@3))^2+4*exp(2*@2)/(exp(2*@1)+exp(2*@2)+exp(2*@3)) ///
	  -4*exp(2*@2+2*@3)/(exp(2*@1)+exp(2*@2)+exp(2*@3))^2)                                  ///
  level(95) ci(probit) 
  disp in smcl in gr "{hline 13}" "{c BT}" "{hline 64}" _n
end

ciherit


******** Continuous neuroticism, P2 ADE, AE, E, CE **************************

use twin_neur, clear

generate k = member if M==0
replace k = pair if M==1

xtmixed neurot || pair: || k:, mle variance

nlcom (var_A: 3*exp(2*[lns1_1_1]_cons) - exp(2*[lns2_1_1]_cons) ) ///
      (var_D: 2*(exp(2*[lns2_1_1]_cons) - exp(2*[lns1_1_1]_cons)))

 * heritability
disp (exp(2*[lns1_1_1]_cons)+exp(2*[lns2_1_1]_cons)) ///
  /(exp(2*[lns1_1_1]_cons)+exp(2*[lns2_1_1]_cons)+exp(2*[lnsig_e]_cons))



* approximate confidence intervals for variance components and heritability
* please check derivatives!
capture program drop _all
program define ciherit
  *ml display, first plus
  _coef_table, first plus
 noi _diparm lns1_1_1 lns2_1_1, label(var_A)                          ///
  func(3*exp(2*@1)-exp(2*@2)) deriv(6*exp(2*@1)  -2*exp(2*@2))        ///
  level(95) ci(log) 
 noi _diparm lns1_1_1 lns2_1_1, label(var_D)                 ///
  func(-2*exp(2*@1)+2*exp(2*@2)) deriv(-4*exp(2*@1)  4*exp(2*@2)) ///
  level(95) ci(log) 
 noi _diparm lnsig_e, label(var_E)                           ///
  func(exp(2*@)) deriv(2*exp(2*@))                           ///
  level(95) ci(log) 
 noi _diparm lns1_1_1 lns2_1_1 lnsig_e, label(heritability)      ///
  func((exp(2*@1)+exp(2*@2))/(exp(2*@1)+exp(2*@2)+exp(2*@3)))              ///
  deriv(-2*exp(2*@1)*(exp(2*@1)+exp(2*@2))/(exp(2*@1)+exp(2*@2)+exp(2*@3))^2+2*exp(2*@1)/(exp(2*@1)+exp(2*@2)+exp(2*@3)) ///
        -2*exp(2*@2)*(exp(2*@1)+exp(2*@2))/(exp(2*@1)+exp(2*@2)+exp(2*@3))^2+2*exp(2*@2)/(exp(2*@1)+exp(2*@2)+exp(2*@3)) ///
	    -2*exp(2*@3)*(exp(2*@1)+exp(2*@2))/(exp(2*@1)+exp(2*@2)+exp(2*@3))^2)                                  ///
  level(95) ci(probit) 
  disp in smcl in gr "{hline 13}" "{c BT}" "{hline 64}" _n
end

ciherit

******* P2 AE
constr def 1 [pair2]_cons = [k1]_cons

gllamm neurot, i(k pair) constr(1) adapt

******* P2 E
xtmixed neurot, mle variance

******* P2 CE
xtmixed neurot || pair:, mle variance





******* Binary hay fever status, P2 ADE, AE **********************************


use twin_hay, clear

generate k = pair if M==1
replace k = member if M==0
generate num3 = freq

gllamm h male, i(k pair) link(probit) fam(binom) adapt weight(num)


nlcom (var_A: 3*[pair2]_cons^2 - [k1]_cons^2) ///
      (var_D: 2*([k1]_cons^2 - [pair2]_cons^2))

*Heritability
display ([k1]_cons^2+[pair2]_cons^2)/([k1]_cons^2+[pair2]_cons^2+1)

* cannot use _diparm  because parameters are not [eqname]_cons


******** P2 AE 

constr def 1 [pair2]_cons = [k1]_cons
gllamm h male, i(k pair) link(probit) fam(binom) ///
   adapt weight(num) constr(1)


   
   
******* Continuous birthweight: Nuclear family data, ACE **********************

use fam_birthwt, clear

list family M F K male birthyr bwt if family<3, sepby(family) noobs

generate var1 = M + K/2
generate var2 = F + K/2
generate var3 = K/sqrt(2)

xtmixed bwt male first midage highage birthyr  || family: var1 var2 var3, ///
  nocons cov(ident) mle variance


**** get heritability and confidence interval

** confidence interval for heritability
** using standard error of heritability

nlcom (heritability: exp(2*[lns1_1_1]_b[_cons]) /  ///
      ( exp(2*[lns1_1_1]_b[_cons]) + exp(2*[lnsig_e]_b[_cons])))

** approximate confidence interval for heritability
** using probit transformation and appropriate standard error 
      
capture program drop _all
program define ciherit
  *ml display, first plus
  _coef_table, first plus
  noi _diparm lnsig_e lns1_1_1, label(heritability)                        ///
  func(exp(2*@2)/(exp(2*@1)+exp(2*@2)))                                    ///
  deriv( -2*exp(2*@1)*exp(2*@2)/(exp(2*@1)+exp(2*@2))^2                    ///
  -2*exp(4*@2)/(exp(2*@1)+exp(2*@2))^2+2*exp(2*@2)/(exp(2*@1)+exp(2*@2)))  ///
  level(95) ci(probit)
  disp in smcl in gr "{hline 13}" "{c BT}" "{hline 64}" _n
end

ciherit

 
